﻿(function($) { 

	// Determine how we should filter
	var camlQuery = "<Query>" +
			"<Where>" +
				"<Geq>" +
					"<FieldRef Name='Modified'/><Value Type='DateTime'><Today OffsetDays='-7'/></Value>" +
				"</Geq>" +
			"</Where>" +
			"<OrderBy>" +
				"<FieldRef Name='Created' Ascending='False'/>" +
			"</OrderBy>" +
		"</Query>";

	// Decide which columns to retrieve: very important with "wide" lists
	var camlViewFields = "<ViewFields>" +
			"<FieldRef Name='Title'/>" +
			"<FieldRef Name='FirstName'/>" +
			"<FieldRef Name='PledgeAmount'/>" +
			"<FieldRef Name='AssignedTo'/>" +
			"<FieldRef Name='Attachments'/>" +
			"<FieldRef Name='Created'/>" +
			"<FieldRef Name='Modified'/>" +
		"</ViewFields>";

	var jsonMapping = {
			ows_ID: {mappedName: "ID", objectType: "Counter"},
			ows_Title: {mappedName: "LastName", objectType: "Text"},
			ows_FirstName: {mappedName: "FirstName", objectType: "Text"},
			ows_PledgeAmount: {mappedName: "PledgeAmount", objectType: "Currency"},
			ows_AssignedTo: {mappedName: "AssignedTo", objectType: "User"}
		};

	var changeToken;

	var me = $().SPServices.SPGetCurrentUser({
		fieldNames: ["ID", "Name", "Title"],
		debug: false
	});



	$(document).ready(function() {

		// Add the output container
		var out = "<table id='fundraiser-pledges-table'>" + formatHeader() + "</table>";
		$("#fundraiser-pledges").html(out);

		// Get the recent items
		getChanges();

		// Set up to check for changes on an interval
		var changesCheck = setInterval(function(){getChanges();}, 10000);

		$("#fundraiser-pledges").on("click", ".pledge-assign input", function() {
			assignToMe($(this).closest("tr"));
		});



	}); // end $(document).ready()


	// functions

	function getChanges() {

		// Go get changes to the list
		var pledgesChangesPromise = $().SPServices({
			operation: "GetListItemChangesSinceToken",
			listName: "Pledges",
			CAMLQuery: camlQuery,
			CAMLViewFields: camlViewFields,
			CAMLQueryOptions: "<QueryOptions>" +
					"<IncludeAttachmentUrls>TRUE</IncludeAttachmentUrls>" +
				"</QueryOptions>",
			changeToken: changeToken
		});


		pledgesChangesPromise.done(function() {

			var pledgeTableBody = $("#fundraiser-pledges-table");
			var thisResult = $(pledgesChangesPromise.responseXML);

			// We need to keep the changeToken so that we can pass it in the next time
			changeToken = thisResult.find("listitems Changes").attr("LastChangeToken");

			// Some of the existing items may have been deleted
			var deletes = thisResult.find("listitems Changes Id[ChangeType='Delete']").each(function() {
				var deleteId = $(this).text();
				var toDelete = pledgeTableBody.find("tr[data-itemid='" + deleteId + "']");
				toDelete.fadeOut();
				toDelete.remove();
			});

			// Process the changes
			var pledges = thisResult.SPFilterNode("z:row").SPXmlToJson({
				mapping: jsonMapping,
				includeAllAttrs: true
			});

			out = "";

			for(var i=0; i < pledges.length; i++) {

				// Format the data for output
				var thisItemData = formatItemData(pledges[i]);

				// Find out if this item already exists on screen
				var existingItem = pledgeTableBody.find("tr[data-itemid='" + pledges[i].ID + "']");
				if(existingItem.length === 0) {
					// Show the new item
					out += thisItemData;
				} else {
					// Update the existing item
					existingItem.replaceWith(thisItemData);
				}

			}
			// Add the new items to the top of the display
			pledgeTableBody.prepend(out);

			// Finally, recalculate the moment timeago displayed for each item
			pledgeTableBody.find(".pledge-created").each(function() {

				var thisCreated = $(this).data("created");
				$(this).html(moment(thisCreated).fromNow());
		
			});

		});


	}
	
	function formatHeader() {
		var out = "<thead>" +
				"<tr class='pledge-header'>" +
					"<th>Pledged By</th>" + 
					"<th>When</th>" +
					"<th>Amount</th>" +
					"<th>Assigned To</th>" +
				"</tr>" +
			"</thead>";
		return out;
	}

	function formatItemData(thisPledge) {

		// Format the fields
		var createdFromNow = moment(thisPledge.Created).fromNow();
		var thisAmount = thisPledge.PledgeAmount !== "" ? parseFloat(thisPledge.PledgeAmount) : "";
		var thisAssignedTo = "";
		if(thisPledge.AssignedTo !== "") {
			// Show a link to the assignee
			thisAssignedTo = showUserLink(thisPledge.AssignedTo.userId, thisPledge.AssignedTo.userName);
		} else if(thisAmount > 100) {
			// Larger donors get personal replies
			thisAssignedTo = "<input type='button' value='Assign to me'>";
		}

		var out = "<tr class='pledge' data-itemid='" + thisPledge.ID + "'>" +
				"<td class='pledge-amount'>" + thisPledge.FirstName + " " + thisPledge.LastName.substr(0, 1) + ".</td>" + 
				"<td class='pledge-created' data-created='" + thisPledge.Created + "'>" + createdFromNow + "</td>" +
				"<td class='pledge-amount'>" + "$" + thisAmount + "</td>" +
				"<td class='pledge-assign'>" + thisAssignedTo + "</td>" +
			"</tr>";
		return out;
	}
	
	function assignToMe(itemRow) {
	
		var thisID = itemRow.data("itemid");
		var assignedTo = me.ID + ";#" + me.Name;

		$().SPServices({
			operation: "UpdateListItems",
			listName: "Pledges",
			ID: thisID,
			valuepairs: [["AssignedTo", assignedTo]],
			completefunc: function (xData, Status) {
				itemRow.find(".pledge-assign").html(showUserLink(me.ID, me.Title));
			}
		});
		
		return false;
	
	}

	function showUserLink(id, name) {
		return "<a href='../_layouts/userdisp.aspx?ID=" + id + "'>" + name + "</a>";	
	}


})(jQuery);